/* chilinfo.c -- report basic information about the first raster in a file
   This program is designed for use in shell-scripts to quickly report some 
   basic properties about a radar file

   Copyright (C) 2000 Robin Hogan
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "chil.h"

void
usage(char *name)
{
  CHILPrintf("Usage:\n"
	     "   %s file.dat [-file|-tape] [-raster] [-date] [-system]\n"
	     "               [-system_id] [-experiment] [-experiment_id]\n"
	     "               [-cloud_data] [-ir]\n", name);
  exit(0);
}


int
main(int argc, char **argv)
{
  ChilboltonFile chil_file;
  RasterInfo raster_info;
  Ray ray;
  int status, file = 2;
  char *space = "";

  if (argc < 2) {
    usage(argv[0]);
  }

  status = CHILOpen(argv[1], &chil_file);
  if (status == FILE_CLOSED) {
    CHILPrintf("Error opening %s\n", argv[1]);
    exit(1);
  }

  status = CHILReadInformationRecord(&chil_file, &raster_info);
  if (status == END_OF_DATA) {
    CHILClose(&chil_file);
    CHILPrintf("Failed to read information record\n");
    exit(1);
  }

  /* Find argument starting with a `-' */
  while (argc > file && *argv[file] != '-') {
    ++file;
  }

  if (argc == file) {
    CHILPrintFileInformation(&chil_file);
    CHILPrintRasterInformation(&raster_info);
    CHILInitRay(&raster_info, 1, 1, &ray);
    status = CHILReadRay(&chil_file, &raster_info, &ray);
   if (status != CHIL_OK) {
      CHILPrintf("Failed to read first ray\n");
    }
    else {
      CHILPrintRayInformation(&ray);
      CHILPrintFieldInformation(&ray);
    }
    CHILClose(&chil_file);
    exit(0);
  }

  for (; argc > file; ++file) {
    if (strcmp(argv[file], "-file") == 0 || strcmp(argv[file], "-tape") == 0) {
      printf("%s%04hu", space, raster_info.file);
      space = " ";
    }
    else if (strcmp(argv[file], "-raster") == 0) {
      printf("%s%hu", space, raster_info.raster);
      space = " ";
    }
    else if (strcmp(argv[file], "-system") == 0) {
      printf("%s%s", space, CHILSystemName(&raster_info, NULL));
      space = " ";
    }
    else if (strcmp(argv[file], "-system_id") == 0) {
      printf("%s%hhu", space, raster_info.system_id);
      space = " ";
    }
    else if (strcmp(argv[file], "-date") == 0) {
      printf("%s%04hu%02hu%02hu", space,
	     CHILTrueYear(raster_info.year), raster_info.month, raster_info.day);
      space = " ";
    }
    else if (strcmp(argv[file], "-experiment") == 0) {
      printf("%s%s", space, CHILExperimentCode(&raster_info, NULL));
      space = " ";
    }
    else if (strcmp(argv[file], "-experiment_id") == 0) {
      printf("%s%hhu", space, raster_info.expt_id);
      space = " ";
    }
    else if (strcmp(argv[file], "-cloud_data") == 0) {
      printf("%s%hhu", space, raster_info.cloud_data);
      space = " ";
    }
    else if (strcmp(argv[file], "-ir") == 0) {
      CHILPrintInformationRecord(&raster_info);
    }
    else {
      printf("%s-", space);
      space = " ";
      CHILPrintf("Warning: `%s' not implemented\n", argv[file]);
    }
  }
  printf("\n");
  CHILClose(&chil_file);
  exit(0);
}
